
#include <dirent.h>
#include <stdio.h>
#include <string.h>
#include <sys/stat.h>
#include <unistd.h>
#include "Error.h"
#include "Client2.h"

Client2::Client2()
{
	strcpy(prefix, "");
}

void Client2::putFile(const char *path)
{
	struct stat statBuf;
	if (stat(path, &statBuf) != 0)
		throw new Error("Cannot stat file \"%s\".", path);

	if (S_ISDIR(statBuf.st_mode))
	{
		DIR *dir = opendir(path);
		if (dir == NULL)
		{
			printf("Cannot open directory \"%n\".\n", path);
			return;
		}

		struct dirent *dirEntry;
		while ((dirEntry = readdir(dir)) != NULL)
		{
			if (strcmp(dirEntry->d_name, ".") == 0
				|| strcmp(dirEntry->d_name, "..") == 0)
				continue;

			char tmpPathName[strlen(path) + 1
				+ strlen(dirEntry->d_name) + 1];
			sprintf(tmpPathName, "%s/%s",
				path, dirEntry->d_name);
			putFile(tmpPathName);
		}

		closedir(dir);

		// make empty dir and/or set the modification time
		char name[4 * 1024];
		sprintf(name, "%s/%s", prefix, path);
		mkdir(name, statBuf.st_mode, statBuf.st_mtime);
		return;
	}

	FILE *inStream = fopen(path, "r");
	if (inStream == NULL)
	{
		printf("Cannot open file \"%s\".\n", path);
		return;
	}

	printf("%s %6.2f%%", path, 0.0);
	fflush(stdout);
	char name[4 * 1024];
	sprintf(name, "%s/%s", prefix, path);
	Client::putFile(name,
		statBuf.st_size, statBuf.st_mode, statBuf.st_mtime,
		inStream);
	fclose(inStream);
	printf("\n");
}

void Client2::putTrack(const char *type, const char *path)
{
	FILE *inStream;
	if (path == NULL
		|| strcmp(path, "-") == 0
		|| strcmp(path, "--stdin") == 0)
		inStream = stdin;
	else
		inStream = fopen(path, "r");

	if (inStream == NULL)
		throw new Error("Cannot open track \"%s\".", path);

	int length = -1;
	if (inStream != stdin)
	{
		struct stat statBuf;
		if (stat(path, &statBuf) == 0
			&& S_ISREG(statBuf.st_mode))
			length = statBuf.st_size;
	}

	if (length > 0)
		printf("%9.2f%%", 0.0);
	else
		printf("%8d KB", 0);
	fflush(stdout);

	int bytesTransmitted = 0;
	while (1)
	{
		char buf[32 * 1024];
		int n = fread(buf, 1, sizeof buf, inStream);
		if (n == 0)
			break;
		Client::putTrack(type, n, buf);
		bytesTransmitted += n;
		if (length > 0)
		{
			printf("\b\b\b\b\b\b\b\b\b\b%9.2f%%",
				100.0 * bytesTransmitted / length);
			fflush(stdout);
		}
		else
		{
			printf("\b\b\b\b\b\b\b\b\b\b\b%8d KB",
				bytesTransmitted / 1024);
			fflush(stdout);
		}
		type = "continue";
	}

	if (inStream != stdin)
		fclose(inStream);

	if (bytesTransmitted == 0)
		throw new Error("Track \"%s\" seems to be empty.", path);
}

void Client2::setPrefix(const char *path)
{
	strcpy(prefix, path);
}
